<?PHP
class DB {
	/**
	 * The variable which holds the singleton instance
	 *
	 * @var object
	 */
	private static $instance;

	/**
	 * The variable which holds the database connection resource
	 *
	 * @var resource
	 */
	private static $link;

	/**
	 * Database name
	 *
	 * @var string
	 */
	private static $database;

	/**
	 * The last query
	 *
	 * @var string
	 */
	private static $lastQuery;

	/**
	 * Constructor returns null, and it's visibility private, referring to singleton pattern
	 */
	private function __construct() {
		return null;
	}

	/**
	 * Attempts to create a connection to the specified database
	 *
	 * @param string $host
	 * @param string $user
	 * @param string $password
	 * @param string $database
	 * @param integer $port
	 * @return void;
	 */
	public static function connect($host, $user, $password, $database, $port = null) {
		if(!extension_loaded('mysqli')) {
			throw new DatabaseException('mysqli extension not loaded');
		}

        if (!isset(self::$instance)) {
            $c = __CLASS__;
            self::$instance = new $c;
        }

		if($port === null) {
			$connection = @mysqli_connect($host, $user, $password);
		} else {
			$connection = @mysqli_connect($host, $user, $password, '', $port);
		}

		if(mysqli_connect_errno()) {
			$error = mysqli_connect_error();
			if($error != '') {
				throw new DatabaseException('connection call failed', $error);
			} else {
				throw new DatabaseException('connection call failed');
			}
		}

		if (!@mysqli_select_db($connection, $database)) {
			switch(mysqli_errno($connection)) {
				case 1049:
					$exc = new DatabaseException("no such database", mysqli_error($connection));
					break;
				case 1044:
					$exc = new DatabaseException("access violation", mysqli_error($connection));
					break;
				default:
					$exc = new DatabaseException("cannot select database", mysqli_error($connection));
			}
			throw $exc;
		}
		self::$link = $connection;
		self::$database = $database;
		self::executeQuery("SET NAMES 'latin2'");
	}

	/**
	 * Closes the current connection
	 *
	 * @return boolean
	 */
	public static function close() {
		$return = mysqli_close(self::$link);
		//unset(self::$link);
		return $return;
	}

	/**
	 * Executes an SQL query (for example a SELECT)
	 *
	 * @param string $sql
	 * @return Database_ResultSet The return value is a Database_ResultSet object
	 */
	public static function executeQuery($sql) {
		self::$lastQuery = $sql;
		$result = @mysqli_query(self::$link, $sql);
		if(!$result) {
			throw new DatabaseException('Could not execute query', mysqli_error(self::$link), $sql);
		}
		return new Database_ResultSet($result);
	}

	/**
	 * Executes an insert SQL query
	 *
	 * @param string $sql
	 * @return integer Returns the last inserted row's ID
	 */
	public static function executeInsert($sql) {
		self::$lastQuery = $sql;
		$result = @mysqli_query(self::$link, $sql);
		if(!$result) {
			throw new DatabaseException('Could not execute insert', mysqli_error(self::$link), $sql);
		}
		return self::getLastInsertID();
	}

	/**
	 * Executes an update, delete SQL query
	 *
	 * @param string $sql
	 * @return integer Return the affected rows number
	 */
	public static function executeUpdate($sql) {
		self::$lastQuery = $sql;
		$result = @mysqli_query(self::$link, $sql);
		if(!$result) {
			throw new DatabaseException('Could not execute update', mysqli_error(self::$link), $sql);
		}
		return self::getUpdateCount();
	}

	/**
	 * Begins the transaction
	 *
	 * @return void
	 */
	public static function beginTransaction() {
		if (!mysqli_autocommit(self::$link, FALSE)) {
			throw new DatabaseException('Could not begin transaction', mysqli_error(self::$link));
		}
	}

	/**
	 * Commits the transaction
	 *
	 * @return void
	 */
	public static function commitTransaction() {
		if (!mysqli_commit(self::$link)) {
			throw new DatabaseException('Can not commit transaction', mysqli_error(self::$link));
		}
		mysqli_autocommit(self::$link, TRUE);
	}

	/**
	 * Rolls back the transaction
	 *
	 * @return void
	 */
	public static function rollbackTransaction() {
		if (!mysqli_rollback(self::$link)) {
			throw new SQLException('Could not rollback transaction', mysqli_error(self::$link));
		}
		mysqli_autocommit(self::$link, TRUE);
	}

	/**
	 * Get affected rows number
	 *
	 * @return integer
	 */
	public static function getUpdateCount() {
		return (int)@mysqli_affected_rows(self::$link);
	}

	/**
	 * Get last inserted row's ID
	 *
	 * @return integer
	 */
	public static function getLastInsertID() {
		return (int)@mysqli_insert_id(self::$link);
	}

	/**
	 * Escapes a string to be safely used in a SQL statement
	 *
	 * @param string $string
	 * @return string
	 */
	public static function escape($string) {
		return mysqli_real_escape_string(self::$link, $string);
	}

	/**
	 * Get the database connection
	 *
	 * @return resource
	 */
	public static function getResource() {
		return self::$link;
	}

	/**
	 * Get the singleton instance
	 *
	 * @return object
	 */
	public static function getInstance() {
		return self::$instance;
	}

}

